# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Running a custom UNIX process outside the GRASS GIS environment
"""
import pickle
from flask import jsonify, make_response

from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.processing.common.ephemeral_custom_processing import (
    start_job,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class AsyncEphemeralCustomResource(ResourceBase):
    """This class represents a custom process resource
    that will be asynchronously executed without the initialization
    of the GRASS environment.
    """

    def __init__(self):
        ResourceBase.__init__(self)

    def post(self, executable):
        """Run a custom process

        The process is identified by executable

        Args:
            executable (str): Name of the executable

        Process arguments must be provided as JSON document in the POST
        request:
            ["arg_1", "arg_2", "arg_3", "arg_4", ...]

        Returns:
            flask.Response:
            The HTTP status and a JSON document that includes the
            status URL of the export task that must be polled for updates.

        """
        # Preprocess the post call
        rdc = self.preprocess(has_json=True)

        if rdc:
            rdc.set_user_data(executable)
            enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Asynchronous computation in specific temporary generated and then copied
or original mapsets
"""
import pickle
from flask import jsonify, make_response
from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import persistent_processing

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.persistent_processing import start_job

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Guido Riembauer, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class AsyncPersistentResource(ResourceBase):
    def __init__(self, resource_id=None, iteration=None, post_url=None):
        ResourceBase.__init__(self, resource_id, iteration, post_url)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("post", persistent_processing.post_doc))
    def post(self, location_name, mapset_name):
        """Execute a user defined process chain that creates a new mapset or
        runs in an existing one.

        The process chain that describes the GRASS modules that should be
        executed must be provided as JSON payload of the POST request.

        Args:
            location_name (str): The name of the location
            mapset_name (str): The name of the mapset

        Returns:
            flask.Response:
            The HTTP status and a JSON document that includes the
            status URL of the task that must be polled for updates.

        Process chain input format:

            {                                           # A process chain is a
                                                          dict with entries for
                                                          each module that
                                                          should be run
               Id:{                                     # Id must be numerical
                                                          and indicates the
                                                          process order
                    "module": <module_name>,            # Name of the module to
                                                          run
                    "stdin":  <Id::stdout | Id::stderr> # Use the output of a
                                                          specific module as
                                                          input for this module
                                                        # Id:stdout, Id:stderr
                                                          are available
                    "inputs ":{                         # Definition of all
                                                          input parameters as
                                                          key:value pairs
                              <parameter name>:<value>, # e.g.: value ==
                                                          "raster_name@mapset_
                                                          name" or "degree" or
                                                          0.0
                              <parameter name>:<value>  # e.g.: value ==
                                                          $file::slope_output_
                                                          file to specify an
                                                          output file
                                                        # that name will be
                                                          automatically
                                                          generated by the API.
                    },
                    "outputs": {                         # Definition of all
                                                           outputs using
                                                           key:value pairs
                        <parameter name>: {
                            "name":<value>,              # Output name e.g.
                                                           "my_aspect_map" or a
                                                           temporary file id
                                                         # definition:
                                                           $file::id eg: $file
                                                           ::aspect_output_file
                                                         # This file can be
                                                           used in other module
                                                           as input
                            "export": {                  # Export options, if
                                                           present this map
                                                           will be exported
                                "format": <value>        # Set the export
                                                           format raster=
                                                           GeoTiff (default),
                                                           vector = shape
                                                           (default)
                                "type": <output type>,   # Output type e.g.:
                                                           raster, vector,
                                                           file, stds
                            }
                        },
                        <parameter name>: {
                            "name": <value>,             # Output name e.g.
                                                           "my_slope_map"
                            "export": {                  # Export options, if
                                                           present this map
                                                           will be exported
                                "format": <value>        # Set the export
                                                           format raster=
                                                           GeoTiff (default),
                                                           vector = shape
                                                           (default)
                                "type": <output type>,   # Output type e.g.:
                                                           raster, vector,
                                                           file, stds
                            }
                        }
                    },
                    "flags": <flags>,                    # All flags in a
                                                           string e.g.: "ge"
                    "overwrite": <True|False>,           # Set True to
                                                           overwrite existing
                                                           data
                    "verbose": <True|False>              # Verbosity of the
                                                           module
                },
               Id: {                                     # The id of an
                                                           executable command,
                                                           that is not a grass
                                                           module
                    "executable": <path>,                # The name and path of
                                                           the executable e.g.
                                                           /bin/cp
                    "stdin": <Id::stdout | Id::stderr>   # Use the output of a
                                                           specific module as
                                                           input for this
                                                           module
                                                         # Id::stdout,
                                                           Id::stderr are
                                                           available
                    "parameters": [<parameter>,]         # A list of strings
                                                           that represent the
                                                           parameters that may
                                                           contain
                                                         # temporary file
                                                           definitions:
                                                           $file::id  eg: $file
                                                           ::aspect_output_file
               },
                ...
            }

        """
        # Preprocess the post call
        rdc = self.preprocess(
            has_json=True, location_name=location_name, mapset_name=mapset_name
        )

        if rdc:
            enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
STRDS raster map layer management

TODO: Integrate into the ephemeral process chain approach
"""

from actinia_core.processing.common.utils import try_import

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


PersistentRasterSTRDSLister = try_import(
    (
        "actinia_core.processing.actinia_processing.persistent"
        + ".strds_raster_management"
    ),
    "PersistentRasterSTRDSLister",
)

PersistentRasterSTRDSRegisterer = try_import(
    (
        "actinia_core.processing.actinia_processing.persistent"
        + ".strds_raster_management"
    ),
    "PersistentRasterSTRDSRegisterer",
)

PersistentRasterSTRDSUnregisterer = try_import(
    (
        "actinia_core.processing.actinia_processing.persistent"
        + ".strds_raster_management"
    ),
    "PersistentRasterSTRDSUnregisterer",
)


def list_raster_strds(*args):
    processing = PersistentRasterSTRDSLister(*args)
    processing.run()


def register_raster(*args):
    processing = PersistentRasterSTRDSRegisterer(*args)
    processing.run()


def unregister_raster(*args):
    processing = PersistentRasterSTRDSUnregisterer(*args)
    processing.run()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Storage base class
"""
from urllib.request import urlopen
from flask.json import loads as json_loads
from flask.json import dumps as json_dumps

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


def get_sentinel_date(product_id):
    """
    Returns year month and day of a (AWS) Sentinel scene from the Sentinel
    product-id.

    Args:
        product_ids (str): The Sentinel product id

    Returns:
        year (str): The recording year of the Sentinel product
        month (str): The recording month of the Sentinel product
        day (str): The recording day of the Sentinel product
    """
    if "S2A_OPER" in product_id:
        year = product_id[63:67]
        month = product_id[67:69]
        day = product_id[69:71]
    else:
        year = product_id[45:49]
        month = product_id[49:51]
        day = product_id[51:53]
    if month != "10":
        month = month.replace("0", "")
    if day not in ["10", "20", "30"]:
        day = day.replace("0", "")
    return year, month, day


class AWSSentinel2AInterface(object):
    """Query interface to the Sentinel2 and Landsat public geo-data
    available in the google cloud storage
    """

    def __init__(self, config):
        """
        Interface to the Sentinel2 and Landsat data in the google cloud storage

        Args:
            config: The configuration of Actinia Core
        """
        self.aws_sentinel_base_url = "http://sentinel-s2-l1c.s3.amazonaws.com"
        self.aws_sentinel_base_eu_central_url = (
            "http://sentinel-s2-l1c.s3-website.eu-central-1.amazonaws.com"
        )
        self.config = config

        self.sentinel_bands = [
            "B01",
            "B02",
            "B03",
            "B04",
            "B05",
            "B06",
            "B07",
            "B08",
            "B8A",
            "B09",
            "B10",
            "B11",
            "B12",
        ]

    def get_sentinel_urls(self, product_ids, bands=None):
        """Receive the download urls and time stamps for a list of Sentinel2
        product ids from AWS service

        1. Transform the Sentinel ID into the path of the productInfo.json url
           that is required to get the tile urls
        2. Parse the productInfo.json file and extract the tile urls
        3. Create the download links for each tile based on each band
        4. Include the tileInfo.json, xml and preview url's

        Args:
            product_ids: A list of sentinel product ids
            bands: A list of band names

        Returns:
            dict
            A dictionary that contains the time stamp, the tile names, the map
            names in GRASS, the aws cloud storage urls, the tile GeoJSON
            footprint and the XML metadata file path download url's.

        Example:

            [{'product_id': 'S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_'
                            '20170202T090155',
              'tiles':[{
                'B04': {
                    'file_name' : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B04.jp2',
                    'map_name'  : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B04',
                    'public_url': 'http://sentinel-s2-l1c.s3.amazonaws.com/'
                                  'tiles/57/V/XE/2015/12/7/0/B04.jp2'},
                'B08': {
                    'file_name' : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B08.jp2',
                    'map_name'  : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B08',
                    'public_url': 'http://sentinel-s2-l1c.s3.amazonaws.com/'
                                  'tiles/57/V/XE/2015/12/7/0/B08.jp2'},
                'info'      : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/tileInfo.json',
                'metadata'  : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/metadata.xml',
                'preview'   : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/preview.jpg',
                'timestamp' : '2015-12-07T00:33:02.634Z',
                'url'       : 'http://sentinel-s2-l1c.s3-website.eu-central-1.'
                              'amazonaws.com/#tiles/57/V/XE/2015/12/7/0/'}]}]

        """
        # Old format
        # products/2015/12/7/S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_
        # V20151207T003302_20151207T003302

        # https://sentinel-s2-l1c.s3.amazonaws.com/products/2015/10/1/S2A_OPER
        # _PRD_MSIL1C_PDMC_20161220T003744_R006_V20151001T071826_20151001T07
        # 1826/productInfo.json

        # New format
        # products/2017/10/31/S2A_MSIL1C_20171031T000721_N0206_R016_T01WCP_
        # 20171031T015145

        #           1         2         3         4         5         6
        # 7         8
        # 0123456789012345678901234567890123456789012345678901234567890
        # 12345678901234567890
        # S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_R102_V20151207T003302
        # _20151207T003302

        #           1         2         3         4         5         6
        # 7         8
        # 0123456789012345678901234567890123456789012345678901234567890
        # 12345678901234567890
        # S2A_MSIL1C_20171031T000721_N0206_R016_T01WCP_20171031T015145

        # Assign default bands
        if bands is None:
            bands = ["B04", "B08"]

        try:
            for band in bands:
                if band not in self.sentinel_bands:
                    raise Exception("Unknown Sentinel-2 band name <%s>" % band)

            result = []

            for product_id in product_ids:
                product_id = product_id.replace(".SAFE", "")

                year, month, day = get_sentinel_date(product_id)

                # Get the product info JSON file
                json_url = (
                    "%(base)s/products/%(year)s/%(month)s/%(day)s/%(id)s/"
                    "productInfo.json"
                    % {
                        "base": self.aws_sentinel_base_url,
                        "year": year,
                        "month": month,
                        "day": day,
                        "id": product_id,
                    }
                )

                response = urlopen(json_url)
                product_info = response.read()

                try:
                    info = json_loads(product_info)
                except Exception:
                    raise Exception(
                        "Unable to read the productInfo.json file from URL: "
                        "%s. Error: %s" % (json_url, product_info)
                    )

                if info:
                    scene_entry = self._parse_scene_info(
                        bands, product_id, info
                    )
                    result.append(scene_entry)

            return result

        except Exception:
            raise

    def _parse_scene_info(self, bands, product_id, info):
        """Helpher method to parse the Sentinel-2 scene info:
        1. Parse the product info and extract the tile urls
        2. Create the download links for each tile based on each band
        3. Include the tileInfo.json, xml and preview url's

        Args:
            bands (list): A list of band names
            product_id (list): The Sentinel product id
            info (dict): A dictionary with the information of the Sentinel
                         product

        Returns:
            scene_entry (dict): A dictionary that contains the time stamp, the
                                tile names, the map names in GRASS, the aws
                                cloud storage urls, the tile GeoJSON footprint
                                and the XML metadata file path download url's.

        Example:

            {'product_id': 'S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_'
                            '20170202T090155',
              'tiles':[{
                'B04': {
                    'file_name' : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B04.jp2',
                    'map_name'  : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B04',
                    'public_url': 'http://sentinel-s2-l1c.s3.amazonaws.com/'
                                  'tiles/57/V/XE/2015/12/7/0/B04.jp2'},
                'B08': {
                    'file_name' : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B08.jp2',
                    'map_name'  : 'S2A_OPER_PRD_MSIL1C_PDMC_20151207T031157_'
                                  'R102_V20151207T003302_20151207T003302_tile_'
                                  '14_band_B08',
                    'public_url': 'http://sentinel-s2-l1c.s3.amazonaws.com/'
                                  'tiles/57/V/XE/2015/12/7/0/B08.jp2'},
                'info'      : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/tileInfo.json',
                'metadata'  : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/metadata.xml',
                'preview'   : 'http://sentinel-s2-l1c.s3.amazonaws.com/tiles/'
                              '57/V/XE/2015/12/7/0/preview.jpg',
                'timestamp' : '2015-12-07T00:33:02.634Z',
                'url'       : 'http://sentinel-s2-l1c.s3-website.eu-central-1.'
                              'amazonaws.com/#tiles/57/V/XE/2015/12/7/0/'}]}

        """
        scene_entry = {}
        scene_entry["product_id"] = product_id
        scene_entry["tiles"] = []
        tile_num = 0
        for tile in info["tiles"]:
            tile_num += 1
            tile_info = {}
            tile_info["timestamp"] = info["timestamp"]

            # http://sentinel-s2-l1c.s3.amazonaws.com/tiles/58/V/CK/2015/12/7/
            # 0/metadata.xml,
            # http://sentinel-s2-l1c.s3.amazonaws.com/tiles/58/V/CK/2015/12/7/
            # 0/tileInfo.json,
            # http://sentinel-s2-l1c.s3.amazonaws.com/tiles/58/V/CK/2015/12/7/
            # 0/preview.jpg

            metadata_url = (
                f'{self.aws_sentinel_base_url}/{tile["path"]}/metadata.xml'
            )
            info_url = (
                f'{self.aws_sentinel_base_url}/{tile["path"]}/tileInfo.json'
            )
            preview_url = (
                f'{self.aws_sentinel_base_url}/{tile["path"]}/preview.jpg'
            )
            tile_info[
                "url"
            ] = f'{self.aws_sentinel_base_eu_central_url}/#{tile["path"]}/'

            tile_info["metadata"] = metadata_url
            tile_info["info"] = info_url
            tile_info["preview"] = preview_url
            public_url = f'{self.aws_sentinel_base_url}/{tile["path"]}'

            for band in bands:
                tile_name = f"{product_id}_tile_{tile_num}_band_{band}.jp2"
                map_name = f"{product_id}_tile_{tile_num}_band_{band}"
                tile_info[band] = {}
                tile_info[band]["file_name"] = tile_name
                tile_info[band]["map_name"] = map_name
                tile_info[band]["public_url"] = f"{public_url}/{band}.jp2"
            scene_entry["tiles"].append(tile_info)
        return scene_entry

    def get_sentinel_tile_footprint(self, tile_entry):
        """
        Downloads the tileInfo.json file and extracts/returns the tile geometry
        aka footprint

        Args:
            tile_entry:

        Returns:
            The geojson string that represents the tile footprint

        """
        # Get the tile info url that contains the tile footprint geojson
        response = urlopen(tile_entry["info"])
        tile_info = response.read()

        try:
            info = json_loads(tile_info)
        except Exception:
            raise Exception(
                "Unable to read the info json file from URL: %s. Error: %s"
                % (tile_entry["info"], tile_info)
            )
        return json_dumps(info["tileDataGeometry"])



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async Process stdout parser test case
"""
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

process_chain = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "5000"},
            ],
            "stdout": {"id": "region", "format": "kv", "delimiter": "="},
            "flags": "g",
        },
        {
            "id": "2",
            "module": "r.out.ascii",
            "inputs": [
                {"param": "input", "value": "elevation@PERMANENT"},
                {"param": "precision", "value": "0"},
            ],
            "stdout": {"id": "elevation", "format": "table", "delimiter": " "},
            "flags": "h",
        },
        {
            "id": "3",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
            "stdout": {"id": "map_list", "format": "list", "delimiter": "\n"},
        },
        {
            "id": "4",
            "module": "r.univar",
            "inputs": [{"param": "map", "value": "elevation@PERMANENT"}],
            "stdout": {"id": "stats", "format": "kv", "delimiter": "="},
            "flags": "g",
        },
        {
            "id": "5",
            "module": "r.univar",
            "inputs": [
                {"param": "map", "value": "elevation@PERMANENT"},
                {"param": "zones", "value": "basin_50K@PERMANENT"},
                {"param": "separator", "value": "pipe"},
            ],
            "stdout": {
                "id": "stats_zonal",
                "format": "table",
                "delimiter": "|",
            },
            "flags": "t",
        },
    ],
}

r_what = {
    "list": [
        {
            "module": "g.region",
            "id": "g_region_1",
            "inputs": [
                {"param": "raster", "value": "landuse96_28m@PERMANENT"}
            ],
            "stdout": {"id": "region", "format": "kv", "delimiter": "="},
            "flags": "ga",
        },
        {
            "module": "r.what",
            "id": "r_what_1",
            "verbose": True,
            "flags": "nfic",
            "inputs": [
                {"param": "map", "value": "landuse96_28m@PERMANENT"},
                {
                    "param": "coordinates",
                    "value": "633614.08,224125.12,632972.36,225382.87",
                },
                {"param": "null_value", "value": "null"},
                {"param": "separator", "value": "pipe"},
            ],
            "stdout": {"id": "sample", "format": "table", "delimiter": "|"},
        },
    ],
    "version": "1",
}


class AsyncProcessStdoutParserTestCase(ActiniaResourceTestCaseBase):
    def test_output_parsing(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        self.assertTrue("region" in resp["process_results"])
        self.assertTrue("elevation" in resp["process_results"])
        self.assertTrue("map_list" in resp["process_results"])
        self.assertTrue("stats" in resp["process_results"])
        self.assertTrue("stats_zonal" in resp["process_results"])
        self.assertTrue("elevation" in resp["process_results"]["map_list"])
        self.assertEqual(resp["process_results"]["stats"]["cells"], "9")
        self.assertEqual(len(resp["process_results"]["elevation"]), 3)
        self.assertEqual(len(resp["process_results"]["region"]), 11)
        self.assertEqual(len(resp["process_results"]["stats_zonal"]), 7)
        self.assertEqual(len(resp["process_results"]["stats_zonal"][0]), 14)

    def test_output_parsing_r_what(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(r_what),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        process_results = {
            "region": {
                "cells": "249324",
                "cols": "526",
                "e": "644971",
                "ewres": "28.5",
                "n": "228527.25",
                "nsres": "28.5",
                "projection": "99",
                "rows": "474",
                "s": "215018.25",
                "w": "629980",
                "zone": "0",
            },
            "sample": [
                [
                    "easting",
                    "northing",
                    "site_name",
                    "landuse96_28m@PERMANENT",
                    "landuse96_28m@PERMANENT_label",
                ],
                ["633614.08", "224125.12", "", "15", "Southern Yellow Pine"],
                [
                    "632972.36",
                    "225382.87",
                    "",
                    "4",
                    "Managed Herbaceous Cover",
                ],
            ],
        }

        self.assertEqual(resp["process_results"], process_results)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Redis base class
"""

import redis
from actinia_core.core.logging_interface import log

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class RedisBaseInterface(object):
    """
    The base class for most redis database interfaces
    """

    def __init__(self):
        self.connection_pool = None
        self.redis_server = None

    def connect(self, host="localhost", port=6379, password=None):
        """Connect to a specific redis server

        Args:
            host (str): The host name or IP address
            port (int): The port
            password (str): The password

        """
        kwargs = dict()
        kwargs["host"] = host
        kwargs["port"] = port
        if password and password is not None:
            kwargs["password"] = password
        self.connection_pool = redis.ConnectionPool(**kwargs)
        del kwargs
        self.redis_server = redis.StrictRedis(
            connection_pool=self.connection_pool
        )
        try:
            self.redis_server.ping()
        except redis.exceptions.ResponseError as e:
            log.error("Could not connect to " + host, port, str(e))
        except redis.exceptions.AuthenticationError:
            log.error("Invalid password")
        except redis.exceptions.ConnectionError as e:
            log.error(str(e))

    def disconnect(self):
        self.connection_pool.disconnect()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Upload raster via endpoint test case
"""
import os
import unittest
import requests

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except Exception:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

__license__ = "GPLv3"
__author__ = "Anika Weinmann, Guido Riembauer"
__copyright__ = "Copyright 2016-2021, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


class UploadRasterLayerTestCase(ActiniaResourceTestCaseBase):
    location = "nc_spm_08"
    mapset = "PERMANENT"
    tmp_mapset = "mapset_upload"
    raster = "elev_ned_30m"
    raster_url = additional_external_data["elev_ned_30m_tif"]
    local_raster = f"/tmp/{raster}.tif"

    ref_info = {
        "cells": "225000",
        "cols": "500",
        "east": "645000",
        "ewres": "30",
        "maptype": "raster",
        "max": "156.3865",
        "min": "55.1736",
        "ncats": "0",
        "north": "228500",
        "nsres": "30",
        "rows": "450",
        "south": "215000",
        "west": "630000",
    }

    @classmethod
    def setUpClass(cls):
        # download a raster to re-upload it
        super(UploadRasterLayerTestCase, cls).setUpClass()

        resp_download = requests.get(cls.raster_url)
        if resp_download.status_code == 200:
            with open(cls.local_raster, "wb") as out:
                for bits in resp_download.iter_content():
                    out.write(bits)

    @classmethod
    def tearDownClass(cls):
        os.remove(cls.local_raster)

    def setUp(self):
        # create new temp mapset
        super(UploadRasterLayerTestCase, self).setUp()
        self.create_new_mapset(self.tmp_mapset, location_name=self.location)

    def tearDown(self):
        # delete mapset
        self.delete_mapset(self.tmp_mapset, location_name=self.location)
        super(UploadRasterLayerTestCase, self).tearDown()

    def test_upload_raster_userdb(self):
        """
        Test successful GeoTIFF upload and check against reference raster info
        """
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.tmp_mapset}"
            f"/raster_layers/{self.raster}"
        )
        multipart_form_data = {"file": open(self.local_raster, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=200,
            status="finished",
        )

        self.assertRasterInfo(
            self.location,
            self.tmp_mapset,
            self.raster,
            self.ref_info,
            self.user_auth_header,
        )

    def test_upload_raster_globaldb_error(self):
        """Test Error if raster is uploaded to global DB"""
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.mapset}/"
            f"raster_layers/{self.raster}"
        )
        multipart_form_data = {"file": open(self.local_raster, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check=(
                f"Mapset <{self.mapset}> exists in the global "
                "dataset and can not be modified."
            ),
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Render base classes
"""

from flask_restful import reqparse
from actinia_core.rest.base.resource_base import ResourceBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


REGION_PARAMETERS = {
    "parameters": [
        {
            "name": "n",
            "description": "Northern border",
            "required": False,
            "in": "query",
            "type": "double",
        },
        {
            "name": "s",
            "description": "Southern border",
            "required": False,
            "in": "query",
            "type": "double",
        },
        {
            "name": "e",
            "description": "Eastern border",
            "required": False,
            "in": "query",
            "type": "double",
        },
        {
            "name": "w",
            "description": "Western border",
            "required": False,
            "in": "query",
            "type": "double",
        },
        {
            "name": "width",
            "description": "Image width",
            "required": False,
            "in": "query",
            "type": "double",
        },
        {
            "name": "height",
            "description": "Image height",
            "required": False,
            "in": "query",
            "type": "double",
        },
    ]
}


class RendererBaseResource(ResourceBase):
    """Base class for render resources"""

    def create_parser(self):
        """Create the render option arguments

        The parameter contain:

            n : for north
            s : for south
            e : for east
            w : for west
            width : for the image width
            height : for the image height
            start_time : start time for STRDS where selection
            end_time : end time for STRDS where selection

        Returns:
            The argument parser

        """
        parser = reqparse.RequestParser()
        parser.add_argument(
            "n",
            type=float,
            location="args",
            help="Northern border must be specified as double value",
        )
        parser.add_argument(
            "e",
            type=float,
            location="args",
            help="Eastern border must be specified as double value",
        )
        parser.add_argument(
            "w",
            type=float,
            location="args",
            help="Western border must be specified as double value",
        )
        parser.add_argument(
            "s",
            type=float,
            location="args",
            help="Southern border must be specified as double value",
        )
        parser.add_argument(
            "width",
            type=float,
            location="args",
            help="Image width must be specified as double value",
        )
        parser.add_argument(
            "height",
            type=float,
            location="args",
            help="Image height must be specified as double value",
        )
        parser.add_argument(
            "start_time",
            type=str,
            location="args",
            help="Start time for STRDS raster map layer selection",
        )
        parser.add_argument(
            "end_time",
            type=str,
            location="args",
            help="Start time for STRDS raster map layer selection",
        )

        return parser

    def create_parser_options(self, args):
        """Parse the request parameters for raster and vector rendering

        By default the width is 800, the height is 600 pixel.

        Args:
            args (dict): The argument dictionary with rendering options

        Returns:
             dict:
             A dictionary of render options or an error response object in an
             case of error. In case of an error response object, an update to
             the resource logger was performed.

        """
        options = {}
        options["width"] = 800
        options["height"] = 600
        if "n" in args and args["n"] is not None:
            options["n"] = args["n"]
        if "s" in args and args["s"] is not None:
            options["s"] = args["s"]
        if "e" in args and args["e"] is not None:
            options["e"] = args["e"]
        if "w" in args and args["w"] is not None:
            options["w"] = args["w"]
        if "width" in args and args["width"] is not None:
            if args["width"] < 1:
                return self.get_error_response(
                    message="Width must be larger than 0"
                )
            if args["width"] > 10000:
                return self.get_error_response(
                    message="Width can not be larger than 10000"
                )
            options["width"] = args["width"]
        if "height" in args and args["height"] is not None:
            if args["height"] < 1:
                return self.get_error_response(
                    message="Height must be larger than 0"
                )
            if args["height"] > 10000:
                return self.get_error_response(
                    "Height can not be larger than 10000"
                )
            options["height"] = args["height"]
        if "start_time" in args and args["start_time"] is not None:
            options["start_time"] = args["start_time"]
        if "end_time" in args and args["end_time"] is not None:
            options["end_time"] = args["end_time"]
        return options



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster colors management

TODO: Maybe more tests required, test_raster_colors.py is in place and works
"""

from tempfile import NamedTemporaryFile
import os
import atexit

from actinia_core.processing.actinia_processing.ephemeral.persistent_processing import (
    PersistentProcessing,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class PersistentRasterColorsRules(PersistentProcessing):
    """Set the color table of a raster map from color rules

    Perform the processing in the original mapset
    """

    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)

    def _execute(self, skip_permission_check=True):
        self._setup()

        raster_name = self.map_name
        self.required_mapsets.append(self.target_mapset_name)

        options = self.request_data

        pc = {"1": {"module": "r.colors", "inputs": {}}}
        pc["1"]["inputs"]["map"] = raster_name + "@" + self.target_mapset_name

        if "rules" in options:
            with NamedTemporaryFile(
                mode="w+",
                delete=False,
                suffix=".color",
                dir=self.temp_file_path,
            ) as rules:
                for line in options["rules"]:
                    rules.write(line + "\n")

            pc["1"]["inputs"]["rules"] = rules.name
            atexit.register(remove_rules_file, rules.name)
        else:
            for option in options:
                pc["1"]["inputs"][option] = options[option]

        process_list = self._validate_process_chain(
            process_chain=pc, skip_permission_check=True
        )
        self._check_lock_target_mapset()
        self._create_temp_database(self.required_mapsets)
        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base,
            mapset_name=self.target_mapset_name,
        )

        self._execute_process_list(process_list)


def remove_rules_file(rules_file):
    os.remove(rules_file)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Geodata download import support test case
"""
from actinia_core.core.geodata_download_importer import (
    GeoDataDownloadImportSupport,
)
from actinia_core.core.common.landsat_processing_library import (
    LandsatProcessing,
)
from actinia_core.core.common.config import global_config
import unittest
import subprocess
import os
import shutil

try:
    from .test_resource_base import additional_external_data
except ModuleNotFoundError:
    from test_resource_base import additional_external_data


__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class MessageDummy(object):
    def info(self, message):
        print(message)

    def warning(self, message):
        print(message)

    def debug(self, message):
        print(message)

    def error(self, message):
        print(message)


def update_dummy(message):
    print(message)


class GeoDataDownloadImportSupportTestCase(unittest.TestCase):
    """This class tests the BigQuery google interface to collect download
    urls of landsat and sentinel data
    """

    cache_dir = "/tmp/temp_test_cache"

    def setUp(self):
        if os.path.isdir(self.cache_dir) is False:
            os.mkdir(self.cache_dir)

    def tearDown(self):
        if os.path.isdir(self.cache_dir) is True:
            shutil.rmtree(self.cache_dir)
        pass

    def _run_process(self, process):
        print("Run process: " + str(process))

        inputlist = list()
        inputlist.append(process.executable)
        inputlist.extend(process.executable_params)

        proc = subprocess.Popen(
            args=inputlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        print("Process id: %i" % proc.pid)

        (stdout_buff, stderr_buff) = proc.communicate()
        print(stdout_buff)
        print(stderr_buff)
        proc.wait()

        self.assertEqual(
            0, proc.returncode, "Error while running %s" % process.executable
        )

    def test_download_commands_gml(self):
        gml = additional_external_data["census_wake2000_gml"]

        gddl = GeoDataDownloadImportSupport(
            config=global_config,
            temp_file_path="/tmp",
            download_cache=self.cache_dir,
            send_resource_update=update_dummy,
            message_logger=MessageDummy(),
            url_list=[
                gml,
            ],
        )

        pl, import_file_info = gddl.get_download_process_list()
        self.assertTrue("xml" in import_file_info[0][0])
        self.assertTrue(len(pl) == 2)

        for p in pl:
            self._run_process(p)

        for url, file_path, file_name in import_file_info:
            gddl.perform_file_validation(filepath=file_name)

        for url, file_path, file_name in import_file_info:
            p = gddl.get_vector_import_command(file_name, "test")
            print(p)
            self.assertEqual("v.import", p.executable)

    def test_download_commands_gml_zip(self):
        url_list = []

        gml = additional_external_data["census_wake2000_gml"]
        gml_zip = additional_external_data["census_wake2000_zip"]
        tif = additional_external_data["geology_30m_tif"]
        tif_zip = additional_external_data["geology_30m_zip"]

        url_list.append(gml_zip)
        url_list.append(tif_zip)
        url_list.append(gml)
        url_list.append(tif)

        gddl = GeoDataDownloadImportSupport(
            config=global_config,
            temp_file_path="/tmp",
            download_cache=self.cache_dir,
            send_resource_update=update_dummy,
            message_logger=MessageDummy(),
            url_list=url_list,
        )

        pl, import_file_info = gddl.get_download_process_list()
        self.assertEqual("application/zip", import_file_info[0][0])
        self.assertTrue(len(pl) == 8)

        for p in pl:
            self._run_process(p)

        for url, file_path, file_name in import_file_info:
            gddl.perform_file_validation(filepath=file_name)

    def test_download_commands_tif(self):
        tif_list = []
        tif_list.append(additional_external_data["geology_30m_tif"])
        tif_list.append(additional_external_data["geology_30m_tif"])
        tif_list.append(additional_external_data["geology_30m_tif"])
        tif_list.append(additional_external_data["geology_30m_tif"])
        tif_list.append(additional_external_data["geology_30m_tif"])
        tif_list.append(additional_external_data["geology_30m_tif"])

        gddl = GeoDataDownloadImportSupport(
            config=global_config,
            temp_file_path="/tmp",
            download_cache=self.cache_dir,
            send_resource_update=update_dummy,
            message_logger=MessageDummy(),
            url_list=tif_list,
        )

        pl, import_file_info = gddl.get_download_process_list()
        self.assertEqual("image/tiff", import_file_info[0][0])
        self.assertEqual("image/tiff", import_file_info[1][0])
        self.assertEqual("image/tiff", import_file_info[2][0])
        self.assertEqual("image/tiff", import_file_info[3][0])
        self.assertEqual("image/tiff", import_file_info[4][0])
        self.assertTrue(len(pl) == 12)

        for p in pl:
            self._run_process(p)

        for url, file_path, file_name in import_file_info:
            gddl.perform_file_validation(filepath=file_name)

        for url, file_path, file_name in import_file_info:
            p = gddl.get_raster_import_command(file_name, "test")
            print(p)
            self.assertEqual("r.import", p.executable)

    def test_landsat_download_commands(self):
        lp = LandsatProcessing(
            config=global_config,
            temp_file_path="/tmp",
            scene_id="LC08_L1GT_001004_20130910_20170502_01_T2",
            download_cache=self.cache_dir,
            send_resource_update=update_dummy,
            message_logger=MessageDummy(),
        )

        pl, import_file_info = lp.get_download_process_list()

        print(import_file_info, len(pl))
        for p in pl:
            print(p)

        pl = lp.get_i_landsat_toar_process_list(atcor_method="DOS4")

        print(import_file_info, len(pl))
        for p in pl:
            print(p)

        pl = lp.get_i_vi_process_list(
            atcor_method="DOS4", processing_method="NDVI"
        )

        print(import_file_info, len(pl))
        for p in pl:
            print(p)

        print(lp.raster_names)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
XML to module list
AW: I suspect that this file is no longer used
    because of F821: undefined names after missing import PyQt4
    PyQt4 is obsolete and since python3 PyQt5 is used
"""

# from PyQt4.QtCore import *
# from PyQt4.QtXml import *
# from config import global_config

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class GrassModuleReader(object):
    """
    Class to parse the module_items.xml file from GRASS

    This class creates a nested module dictionary based on keywords.
    The keyword depth is 2, if the second keyword is missing, the
    keyword "unspecific" is used as default.
    """

    def __init__(self, filename):
        self.modules_keyword_dict = {}

        file = QFile(filename)
        file.open(QIODevice.ReadOnly)
        self.xml = QXmlStreamReader(file)

        while not self.xml.atEnd():
            self.xml.readNext()
            if self.xml.isStartElement():
                if self.xml.name() == "module-item":
                    name = self.xml.attributes().value("name")
                    if name:
                        self.parse_module()

    def get(self):
        """
        Return the keyword dictionary
        :return:
        """
        return self.modules_keyword_dict

    def parse_module(self):
        module = None
        keywords = None
        description = None

        while self.xml.readNextStartElement():
            if self.xml.name() == "module":
                module = self.xml.readElementText()
            elif self.xml.name() == "description":
                description = self.xml.readElementText()
            elif self.xml.name() == "keywords":
                keywords = self.xml.readElementText()

        if module and keywords and description:
            if module in global_config.MODULE_ALLOW_LIST:
                # First and second keyword must exist
                key_list = str(keywords).split(",")
                first_key = key_list[0]
                second_key = "unspecific"
                if len(key_list) > 1:
                    second_key = key_list[1]

                if first_key not in self.modules_keyword_dict:
                    self.modules_keyword_dict[first_key] = {}
                if second_key not in self.modules_keyword_dict[first_key]:
                    self.modules_keyword_dict[first_key][second_key] = {}

                module_entry = self.modules_keyword_dict[first_key][second_key]
                module_entry[str(module)] = {}
                module_entry[str(module)]["module"] = str(module)
                module_entry[str(module)]["description"] = str(description)
                module_entry[str(module)]["keywords"] = str(keywords)


def test():
    url = global_config.GRASS_MODULES_XML_PATH
    xml = GrassModuleReader(url)

    for first_key in xml.modules_keyword_dict:
        print(first_key)
        level_one = xml.modules_keyword_dict[first_key]
        for second_key in level_one:
            print("  " + second_key)
            level_two = level_one[second_key]
            for module in level_two:
                entry = level_two[module]
                print("    " + entry["module"])
                print("      " + entry["description"])


def main():
    grass_modules_xml_path = global_config.GRASS_MODULES_XML_PATH
    module_reader = GrassModuleReader(grass_modules_xml_path)

    file = open("grass_modules_list.py", "w")
    file.write("module_list=")
    file.write(str(module_reader.get()))
    file.close()


if __name__ == "__main__":
    test()
    main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Asynchronous computation in specific temporary generated and then copied
or original mapsets
"""
import os
import shutil
import subprocess

from actinia_core.processing.actinia_processing.ephemeral_processing import (
    EphemeralProcessing,
)
from actinia_core.core.common.exceptions import AsyncProcessError
from actinia_core.core.mapset_merge_utils import change_mapsetname

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Guido Riembauer, Anika Weinmann, Lina Krisztian"
__copyright__ = (
    "Copyright 2016-2023, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class PersistentProcessing(EphemeralProcessing):
    """Processing of grass modules in a temporary or original mapset.

    This class is designed to run GRASS modules that are specified in a
    process chain in a temporary mapset that later on is copied into the
    original location or merged into an existing mapset.

    Locking concept:

        - Check if the target mapset exists
        - Lock the target mapset
        - Create a temporary mapset lock (name is generated in constructor)
        - Create the temporary mapset in the local storage
        - Process

    If target mapset exists:

        - Move the temporary mapset after processing to the user group database
        - Merge the temporary mapset into the target mapset
        - Delete the temporary mapset
        - Unlock the two mapsets after processing is finished, terminated or
          raised an error

    If target mapset does not exists:

        - After processing finished successfully, copy the
          temporary mapset to the original user group specific location using
          the target mapset name
        - Unlock the two mapsets after processing is finished, terminated or
          raised an error

    """

    def __init__(self, rdc):
        """Constructor

        Args:
            rdc (ResourceDataContainer): The data container that contains all
                                         required variables for processing

        """

        EphemeralProcessing.__init__(self, rdc)
        self.target_mapset_name = self.mapset_name
        self.target_mapset_exists = False  # By default the target mapset
        # does not exists
        # Set True if this process was successful in setting the lock
        self.target_mapset_lock_set = False
        self.orig_mapset_path = None

        # We have two mapset lock ids. The target mapset and the temporary
        # mapset
        self.target_mapset_lock_id = self._generate_mapset_lock_id(
            self.user_group, self.location_name, self.target_mapset_name
        )

        self.temp_mapset_lock_id = self._generate_mapset_lock_id(
            self.user_group, self.location_name, self.temp_mapset_name
        )
        self.temp_mapset_lock_set = False

    def _generate_mapset_lock_id(self, user_group, location_name, mapset_name):
        """Generate a unique id to lock a mapset in the redis database

        Locations are user group specific. Hence different user groups may have
        locations with the same names and with equal mapset names.

        In the same user group, a location/mapset must be locked to grant
        exclusive access rights.

        Args:
            user_group: The user group used for locking
            location_name: The location name in which the mapset is located
                           for locking
            mapset_name: The mapset name that should be locked

        Returns:
            The lock id

        """
        return "%s/%s/%s" % (user_group, location_name, mapset_name)

    def _lock_temp_mapset(self):
        """Lock the temporary mapset

        This method sets in case of success: self.tmp_mapset_lock_set = True
        """
        # Lock the temporary mapset for the time that the user can allocate at
        # maximum
        ret = self.lock_interface.lock(
            resource_id=self.temp_mapset_lock_id,
            expiration=self.process_time_limit * self.process_num_limit,
        )

        if ret == 0:
            raise AsyncProcessError(
                "Unable to lock temporary mapset <%s>, "
                "resource is already locked" % self.target_mapset_name
            )
        self.message_logger.info(
            "Mapset <%s> locked" % self.target_mapset_name
        )

        # if we manage to come here, the lock was correctly set
        self.temp_mapset_lock_set = True

    def _check_mapset(self, mapset):
        """Check if the target mapset exists

        This method will check if the target mapset exists in the global and
        user group locations.
        If the mapset is in the global database, then an AsyncProcessError
        will be raised, since global location/mapsets can not be modified.

        This method sets in case of success:

            self.target_mapset_lock_set = True
            self.tmp_mapset_lock_set = True

        Raises:
            AsyncProcessError

        """
        mapset_exists = False

        # Check if the global location is accessible and that the target mapset
        # does not exist
        if self.is_global_database is True:
            # Break if the target mapset exists in the global database
            if (
                os.path.exists(self.global_location_path)
                and os.path.isdir(self.global_location_path)
                and os.access(
                    self.global_location_path, os.R_OK | os.X_OK | os.W_OK
                )
                is True
            ):
                self.orig_mapset_path = os.path.join(
                    self.global_location_path, mapset
                )

                if os.path.exists(self.orig_mapset_path) is True:
                    if (
                        os.access(
                            self.orig_mapset_path, os.R_OK | os.X_OK | os.W_OK
                        )
                        is True
                    ):
                        raise AsyncProcessError(
                            "Mapset <%s> exists in the global "
                            "dataset and can not be modified." % mapset
                        )
            else:
                raise AsyncProcessError(
                    "Unable to access global location <%s>"
                    % self.location_name
                )

        # Always check if the target mapset already exists and set the flag
        # accordingly
        if (
            os.path.exists(self.user_location_path)
            and os.path.isdir(self.user_location_path)
            and os.access(self.user_location_path, os.R_OK | os.X_OK | os.W_OK)
            is True
        ):
            self.orig_mapset_path = os.path.join(
                self.user_location_path, mapset
            )

            if os.path.exists(self.orig_mapset_path) is True:
                if (
                    os.access(
                        self.orig_mapset_path, os.R_OK | os.X_OK | os.W_OK
                    )
                    is True
                ):
                    mapset_exists = True
                    # Add the existing mapset to the required ones for mapset
                    # search path settings
                    self.required_mapsets.append(mapset)
                else:
                    raise AsyncProcessError(
                        "Unable to access mapset <%s> "
                        "path %s" % (mapset, self.orig_mapset_path)
                    )
            else:
                mapset_exists = False
        else:
            raise AsyncProcessError(
                "Unable to access user location <%s>" % self.location_name
            )

        return mapset_exists

    def _check_target_mapset_exists(self):
        """Check if the target mapset exists

        This method will check if the target mapset exists in the global and
        user location.
        If the mapset is in the global database, then an AsyncProcessError will
        be raised, since global mapsets can not be modified.

        This method sets in case of success:

            self.target_mapset_exists = True/False

        Raises:
            AsyncProcessError
        """
        self.target_mapset_exists = self._check_mapset(self.target_mapset_name)

    def _check_lock_target_mapset(self):
        """
        Check if the target mapset exists and lock it, then lock the temporary
        mapset

        This method will check if the target mapset exists in the global and
        user location.
        If the mapset is in the global database, then an AsyncProcessError will
        be raised, since global mapsets can not be modified.

        This method sets in case of success:

            self.target_mapset_lock_set = True
            self.tmp_mapset_lock_set = True

        Raises:
            AsyncProcessError

        """
        self._check_target_mapset_exists()
        self._lock_target_mapset()

    def _lock_target_mapset(self):
        """Lock the target mapset

        Raises:
            AsyncProcessError

        """

        # Lock the mapset for the time that the user can allocate at maximum
        ret = self.lock_interface.lock(
            resource_id=self.target_mapset_lock_id,
            expiration=self.process_time_limit * self.process_num_limit,
        )

        if ret == 0:
            raise AsyncProcessError(
                "Unable to lock location/mapset <%s/%s>, "
                "resource is already locked"
                % (self.location_name, self.target_mapset_name)
            )
        self.message_logger.info(
            "location/mapset <%s/%s> locked"
            % (self.location_name, self.target_mapset_name)
        )

        # if we manage to come here, the lock was correctly set
        self.target_mapset_lock_set = True

    def _merge_mapset_into_target(self, source_mapset, target_mapset):
        """Link the source mapset content into the target mapset

        Attention: Not all directories and files in the mapset are copied.
            See list directories.
        """
        self.message_logger.info(
            "Copy source mapset <%s> content "
            "into the target mapset <%s>" % (source_mapset, target_mapset)
        )

        # Raster, vector, group and space time data set directories/files
        directories = [
            "cell",
            "misc",
            "fcell",
            "cats",
            "cellhd",
            "cell_misc",
            "colr",
            "colr2",
            "hist",
            "vector",
            "group",
            "tgis",
            "VAR",
        ]

        for directory in directories:
            source_path = os.path.join(
                self.user_location_path, source_mapset, directory
            )
            target_path = os.path.join(self.user_location_path, target_mapset)

            change_mapsetname(
                source_path,
                directory,
                source_mapset,
                target_mapset,
                target_path,
            )

            if os.path.exists(source_path) is True:
                # Hardlink the sources into the target
                stdout = subprocess.PIPE
                stderr = subprocess.PIPE

                p = subprocess.Popen(
                    [
                        "/bin/cp",
                        "-flr",
                        "%s" % source_path,
                        "%s/." % target_path,
                    ],
                    stdout=stdout,
                    stderr=stderr,
                )
                (stdout_buff, stderr_buff) = p.communicate()
                if p.returncode != 0:
                    raise AsyncProcessError(
                        "Unable to merge mapsets. Error in linking:"
                        " stdout: %s stderr: %s" % (stdout_buff, stderr_buff)
                    )

    def _copy_merge_tmp_mapset_to_target_mapset(self):
        """Copy the temporary mapset into the original location

        In case the mapset does not exists, then use the target mapset name,
        otherwise use the temporary mapset name for copying which is later on
        merged into the target mapset and then removed
        """

        # Extent the mapset lock for an hour, since copying can take long
        if self.target_mapset_lock_set is True:
            ret = self.lock_interface.extend(
                resource_id=self.target_mapset_lock_id, expiration=3600
            )
            if ret == 0:
                raise AsyncProcessError(
                    "Unable to extend lock for mapset "
                    "<%s>" % self.target_mapset_name
                )

        if self.temp_mapset_lock_set is True:
            ret = self.lock_interface.extend(
                resource_id=self.temp_mapset_lock_id, expiration=3600
            )
            if ret == 0:
                raise AsyncProcessError(
                    "Unable to extend lock for "
                    "temporary mapset <%s>" % self.temp_mapset_name
                )

        self.message_logger.info(
            "Copy temporary mapset from %s to %s"
            % (
                self.temp_mapset_path,
                os.path.join(self.user_location_path, self.target_mapset_name),
            )
        )

        source_path = self.temp_mapset_path

        # In case the mapset does not exists, then use the target mapset name,
        # otherwise use the temporary mapset name for copying which is later
        # on merged into the target mapset and then removed
        if self.target_mapset_exists is True:
            target_path = self.user_location_path + "/."
            message = (
                "Copy temporary mapset <%s> to target location "
                "<%s>" % (self.temp_mapset_name, self.location_name)
            )
        else:
            target_path = os.path.join(
                self.user_location_path, self.target_mapset_name
            )
            message = (
                "Copy temporary mapset <%s> to target location "
                "<%s>" % (self.target_mapset_name, self.location_name)
            )

        self._send_resource_update(message)

        try:
            stdout = subprocess.PIPE
            stderr = subprocess.PIPE
            p = subprocess.Popen(
                ["/bin/cp", "-fr", "%s" % source_path, "%s" % target_path],
                stdout=stdout,
                stderr=stderr,
            )
            (stdout_buff, stderr_buff) = p.communicate()
            if p.returncode != 0:
                raise AsyncProcessError(
                    "Unable to copy temporary mapset to "
                    "original location. Copy error "
                    "stdout: %s stderr: %s returncode: %i"
                    % (stdout_buff, stderr_buff, p.returncode)
                )
        except Exception as e:
            raise AsyncProcessError(
                "Unable to copy temporary mapset to "
                "original location. Exception %s" % str(e)
            )

        # Merge the temp mapset into the target mapset in case the target
        # already exists
        if self.target_mapset_exists is True:
            self._merge_mapset_into_target(
                self.temp_mapset_name, self.target_mapset_name
            )
            shutil.rmtree(
                os.path.join(self.user_location_path, self.temp_mapset_name)
            )
            # remove interim results
            if self.interim_result.saving_interim_results is True:
                interim_dir = os.path.join(
                    self.interim_result.user_resource_interim_storage_path,
                    self.resource_id,
                )
                self.message_logger.info(
                    "Remove interim results %s" % interim_dir
                )
                if os.path.isdir(interim_dir):
                    shutil.rmtree(interim_dir)

    def _execute_process_list(self, process_list):
        """Extend the mapset lock and execute the provided process list

        Args:
            process_list: The process list to execute

        Raises:
            This method will raise an AsyncProcessError or
            AsyncProcessTermination
        """
        for process in process_list:
            # Extent the lock for each process by max processing time * 2
            if self.target_mapset_lock_set is True:
                ret = self.lock_interface.extend(
                    resource_id=self.target_mapset_lock_id,
                    expiration=self.process_time_limit * 2,
                )
                if ret == 0:
                    raise AsyncProcessError(
                        "Unable to extend lock for mapset <%s>"
                        % self.target_mapset_name
                    )

            if self.temp_mapset_lock_set is True:
                # Extent the lock for each process by max processing time * 2
                ret = self.lock_interface.extend(
                    resource_id=self.temp_mapset_lock_id,
                    expiration=self.process_time_limit * 2,
                )
                if ret == 0:
                    raise AsyncProcessError(
                        "Unable to extend lock for "
                        "temporary mapset <%s>" % self.temp_mapset_name
                    )

            if process.exec_type == "grass":
                self._run_module(process)
            elif process.exec_type == "exec":
                self._run_process(process)
            elif process.exec_type == "python":
                eval(process.executable)

    def _execute(self, skip_permission_check=False):
        """Overwrite this function in subclasses

        - Call self._setup()
        - Analyse the process chain
        - Check the target mapset and lock it for the maximum time
          a user can consume -> process_num_limit*process_time_limit
        - Initialize and create the temporal database and mapset
          or use the original mapset
        - Run the modules and extend the lock each run
        - Copy the mapset if it has not already exist
        - Cleanup and unlock the mapset

        """

        # Setup the user credentials and logger
        self._setup()
        # check if this is a job resumption
        if self.rdc.iteration is not None:
            # Create the process chain
            (
                pc_step,
                old_process_chain_list,
            ) = self._get_previous_iteration_process_chain()
            self.interim_result.set_old_pc_step(pc_step)
            process_list = self._validate_process_chain(
                process_chain=self.request_data,
                old_process_chain=old_process_chain_list,
                pc_step=pc_step,
            )
            # check iterim results
            (
                interim_result_mapset,
                interim_result_file_path,
            ) = self.interim_result.check_interim_result_mapset(
                pc_step, self.rdc.iteration - 1
            )
        else:
            # Create the process chain
            process_list = self._validate_process_chain()
            interim_result_mapset = None
            interim_result_file_path = None

        # Check and lock the target and temp mapsets
        self._check_lock_target_mapset()

        if self.target_mapset_exists is False:
            # Create the temp database and link the
            # required mapsets into it
            self._create_temp_database(self.required_mapsets)

            # Initialize the GRASS environment and switch into PERMANENT
            # mapset, which is always linked
            self._create_grass_environment(
                grass_data_base=self.temp_grass_data_base,
                mapset_name="PERMANENT",
            )

            # Create the temporary mapset with the same name as the target
            # mapset and switch into it
            self._create_temporary_mapset(
                temp_mapset_name=self.target_mapset_name,
                interim_result_mapset=interim_result_mapset,
                interim_result_file_path=interim_result_file_path,
            )
            self.temp_mapset_name = self.target_mapset_name
        else:
            # Init GRASS environment and create the temporary mapset
            self._create_temporary_grass_environment(
                source_mapset_name=self.target_mapset_name
            )
            self._lock_temp_mapset()

        # Execute the process list
        self._execute_process_list(process_list)
        # Copy local mapset to original location, merge mapsets
        # if necessary
        self._copy_merge_tmp_mapset_to_target_mapset()
        # Parse the module sdtout outputs and create the results
        self._parse_module_outputs()

    def _final_cleanup(self):
        """
        Final cleanup called in the run function at the very end of processing
        """
        # Clean up and remove the temporary gisdbase
        self._cleanup()
        # Unlock the mapsets
        if self.target_mapset_lock_set is True:
            self.lock_interface.unlock(self.target_mapset_lock_id)
        if self.temp_mapset_lock_set is True:
            self.lock_interface.unlock(self.temp_mapset_lock_id)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster colors management

TODO: Maybe more tests required, test_raster_colors.py is in place and works
"""

from flask_restful_swagger_2 import swagger
from flask import jsonify, make_response
from actinia_api.swagger2.actinia_core.apidocs import raster_colors

from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.raster_colors import start_job_colors_out
from actinia_core.processing.common.raster_colors import start_job_from_rules

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class SyncPersistentRasterColorsResource(ResourceBase):
    """Manage the color table"""

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", raster_colors.get_doc))
    def get(self, location_name, mapset_name, raster_name):
        """Get the color definition of an existing raster map layer.

        Args:
            location_name: Name of the location
            mapset_name: Name of the mapset
            raster_name: name of the raster map

        """
        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=raster_name,
        )

        enqueue_job(
            self.job_timeout,
            start_job_colors_out,
            rdc,
            queue_type_overwrite=True,
        )
        http_code, response_model = self.wait_until_finish()
        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("post", raster_colors.post_doc))
    def post(self, location_name, mapset_name, raster_name):
        """Set the color definition for an existing raster map layer.

        The JSON input should contain the color rules, a predefined color table
        or a raster layer:

            {"rules":["1 0:0:0", "default 255:255:255"]}
            {"color":"gbr"}
            {"raster":"elevation"}


        Args:
            location_name: Name of the location
            mapset_name: Name of the mapset
            raster_name: name of the raster map

        Raises:
            InvalidUsage in case the color definition is false

        """
        rdc = self.preprocess(
            has_json=True,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=raster_name,
        )

        if isinstance(self.request_data, dict) is False:
            return self.get_error_response(
                message="Missing raster color definition in JSON content"
            )

        if len(self.request_data) > 1:
            return self.get_error_response(
                message="The color settings expect a single entry"
            )

        if "rules" in self.request_data:
            if isinstance(self.request_data["rules"], list) is False:
                return self.get_error_response(
                    message="Wrong rules definitions in JSON content"
                )
        elif "color" in self.request_data:
            pass
        elif "raster" in self.request_data:
            if "@" not in self.request_data["raster"]:
                return self.get_error_response(
                    message="Missing mapset definition in raster name"
                )
        else:
            return self.get_error_response(
                message="Missing raster color definitions in JSON content"
            )

        enqueue_job(self.job_timeout, start_job_from_rules, rdc)
        http_code, response_model = self.wait_until_finish()
        return make_response(jsonify(response_model), http_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2021 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Utils file with common util functions
"""

import os
from actinia_core.core.common.process_object import Process
from actinia_core.core.common.exceptions import SecurityError
from actinia_core.core.common.config import global_config

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


def ensure_valid_path(path_parts, intent="r"):
    """
    The function returns a joined and normalized path or raises an error if
    it is not as expected for the passed intent.

    Args:
        - path_parts (list of strings): The parts of the path in a list
        - intent (string): for what you want the path to be ensured, can
            be "r" (read), "w" (write) or "rw" (read and write). Default
            is to check for read access.

    Returns:
        path: joined normalized path

    Raise:
        - raises a TypeError if the intent is not 'r', 'w' or 'rw'
        - raises a SecurityError if the path is not valid for the given intent
    """
    allowed_intents = ["r", "w", "rw"]
    if intent not in allowed_intents:
        raise TypeError(
            f"Intent '{intent}' not allowed, allowed are 'r', 'w' and 'rw'"
        )

    path = os.path.normpath(os.path.join(*path_parts))
    error_message = f"Path '{path}' with intent '{intent}' is insecure."

    if not path.startswith(path_parts[0]):
        raise SecurityError(f"{error_message} Too deep path traversal?")
    if intent == "r" and not path_is_in_allowed_read_paths(path):
        raise SecurityError(error_message)
    if intent == "w" and not path_is_in_allowed_write_paths(path):
        raise SecurityError(error_message)
    if intent == "rw" and not path_is_in_allowed_read_and_write_paths(path):
        raise SecurityError(error_message)

    return path


def path_is_in_allowed_read_and_write_paths(path):
    """Checks whether the passed path is a valid read and write path, uses the
    global_config variables `GRASS_USER_DATABASE` and `GRASS_RESOURCE_DIR`
    for the check.

    Args:
        path (string): a normalized path

    Returns:
        (bool) Whether the passed path is a valid write path
    """
    read_allowed = path_is_in_allowed_read_paths(path)
    both_allowed = read_allowed and path_is_in_allowed_write_paths(path)
    return both_allowed


def path_is_in_allowed_read_paths(path):
    """Checks whether the passed path is a valid read path, uses the
    global_config variables `GRASS_USER_DATABASE`, `GRASS_DATABASE` and
    `GRASS_RESOURCE_DIR` for the check.

    Args:
        path (string): a normalized path

    Returns:
        (bool) Whether the passed path is a valid write path
    """
    allowed_read_starts = [
        global_config.GRASS_USER_DATABASE,
        global_config.GRASS_DATABASE,
        global_config.GRASS_RESOURCE_DIR,
    ]
    return any(path.startswith(allowed) for allowed in allowed_read_starts)


def path_is_in_allowed_write_paths(path):
    """Checks whether the passed path is a valid write path, uses the
    global_config variables `GRASS_USER_DATABASE` and `GRASS_RESOURCE_DIR` for
    the check.

    Args:
        path (string): a normalized path

    Returns:
        (bool) Whether the passed path is a valid write path
    """
    allowed_write_starts = [
        global_config.GRASS_USER_DATABASE,
        global_config.GRASS_RESOURCE_DIR,
    ]
    return any(path.startswith(allowed) for allowed in allowed_write_starts)


def get_wget_process(source, url):
    """The function returns a wget Process for the given source and url

    Args:
        source (str): The source file name where to download the data
        url (str): The url which should be downloaded

    Returns:
        p (Process): process for wget
    """

    wget = "/usr/bin/wget"
    wget_params = list()
    wget_params.append("-t5")
    wget_params.append("-c")
    wget_params.append("-q")
    wget_params.append("-O")
    wget_params.append("%s" % source)
    wget_params.append(url)

    p = Process(
        exec_type="exec",
        executable=wget,
        executable_params=wget_params,
        id=f"importer_wget_{os.path.basename(source)}",
        skip_permission_check=True,
    )
    return p


def get_mv_process(source, dest):
    """The function returns a move Process for the given source and dest

    Args:
        source (str): The source file name
        dest (str): The destination file name

    Returns:
        p (Process): process for mv
    """

    copy = "/bin/mv"
    copy_params = list()
    copy_params.append(source)
    copy_params.append(dest)

    p = Process(
        exec_type="exec",
        executable=copy,
        executable_params=copy_params,
        id=f"importer_mv_{os.path.basename(source)}",
        skip_permission_check=True,
    )
    return p


def allowed_file(filename, allowed_extensions):
    """The function checks if the file has an allowed extension.

    Args:
        filename (str): The file name
        allowed_extensions (list): The list of allowed extensions

    Returns:
        (bool): Returns True, if file extension is allowed
    """
    return (
        "." in filename
        and filename.rsplit(".", 1)[1].lower() in allowed_extensions
    )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: API logging test case
"""
import unittest
from actinia_core.core.common.api_logger import ApiLogger
from actinia_core.core.common.app import flask_app

try:
    from .test_resource_base import ActiniaResourceTestCaseBase
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class DummyHTTPRequest(object):
    def __init__(self):
        self.path = "path"
        self.endpoint = "endpoint"
        self.url = "url"
        self.method = "GET"


class ApiLoggingTestCase(ActiniaResourceTestCaseBase):
    """
    This class tests the api logging interface
    """

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()
        # The test user
        self.user_id = "soeren"
        self.log = ApiLogger()
        self.request_object = DummyHTTPRequest()

    def tearDown(self):
        self.app_context.pop()

    def test_logging(self):
        ret = self.log.add_entry(
            user_id=self.user_id, http_request=self.request_object
        )

        self.assertTrue(ret, "add_entry does not work")

        ret = self.log.add_entry(
            user_id=self.user_id, http_request=self.request_object
        )

        self.assertTrue(ret, "add_entry does not work")

        ret = self.log.add_entry(
            user_id=self.user_id, http_request=self.request_object
        )

        self.assertTrue(ret, "add_entry does not work")

        size = self.log.size(self.user_id)

        self.assertEqual(size, 3, "The size method does not work %i" % size)

        list_of_logs = self.log.list(self.user_id, start=0, end=-1)

        for entry in list_of_logs:
            self.assertEqual(entry["api_info"]["endpoint"], "endpoint")
            self.assertEqual(entry["api_info"]["method"], "GET")

        # Remove the first entry
        self.log.trim(self.user_id, 1, -1)

        size = self.log.size(self.user_id)

        self.assertEqual(size, 2, "The size method does not work %i" % size)

        list_of_logs = self.log.list(self.user_id, start=0, end=-1)

        for entry in list_of_logs:
            self.assertEqual(entry["api_info"]["endpoint"], "endpoint")
            self.assertEqual(entry["api_info"]["method"], "GET")

        self.assertTrue(self.log.delete(self.user_id))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2023 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
App configuration of flask, flask_restful, redis server connection
and global settings
"""
actinia_string = """Actinia"""

actinia_description = """
================================
Actinia - The GRASS GIS REST API
================================

**Actinia** is an open source REST API for scalable, distributed, high
performance processing of geographical data that uses GRASS GIS for
computational tasks.

It provides a REST API to process satellite images, time series of satellite
images, arbitrary raster data with geographical relations and vector data.

The REST interface allows to access, manage and manipulate the GRASS GIS
database via HTTP GET,PUT,POST and DELETE requests and to process raster,
vector and time series data located in a persistent GRASS GIS database.
**Actinia** allows the processing of cloud based data, for example all Landsat
4-8 scenes as well as all Sentinel2A scenes in an ephemeral databases. The
computational results of ephemeral processing are available via object storage
as GeoTIFF files.

The full API documentation is available here:
https://redocly.github.io/redoc/?url=https://actinia.mundialis.de/latest/
swagger.json


Examples:
---------

To execute the examples, first setup login information, IP address and port:

    export ACTINIA_URL=https://actinia.mundialis.de/latest
    export AUTH='-u demouser:gu3st!pa55w0rd'

**Data management**

- List all locations that are available in the actinia persistent database:

    curl ${AUTH} -X GET "${ACTINIA_URL}/locations"

- List all mapsets in the location latlong_wgs84:

    curl ${AUTH} -X GET "${ACTINIA_URL}/locations/latlong_wgs84/mapsets"

- List all raster layers in location latlong_wgs84 and mapset Sentinel2A

    curl ${AUTH} -X GET \
    "${ACTINIA_URL}/locations/latlong_wgs84/mapsets/Sentinel2A/raster_layers"

- List all space-time raster datasets (STRDS) in location ECAD and mapset
  PERMANENT:

    curl ${AUTH} -X GET \
    "${ACTINIA_URL}/locations/ECAD/mapsets/PERMANENT/raster_layers"

- List all raster map layers of the STRDS precipitation_1950_2013_yearly_mm:

    curl ${AUTH} -X GET \
    "${ACTINIA_URL}/locations/ECAD/mapsets/PERMANENT/strds/precipitation_\
    1950_2013_yearly_mm/raster_layers"

**Landsat and Sentinel2A NDVI computation**

This API call will compute the NDVI of the top of atmosphere (TOAR)
corrected Landsat4 scene LC80440342016259LGN00:

    curl ${AUTH} -X POST "${ACTINIA_URL}/landsat_process/\
    LC80440342016259LGN00/TOAR/NDVI"

NDVI computation of Sentinel2A scene
S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138:

    curl ${AUTH} -X POST "${ACTINIA_URL}/sentinel2_process/ndvi/\
    S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138"

The results of the asynchronous computations are available as GeoTIFF file in
a cloud storage for download.
"""

from flask_httpauth import HTTPBasicAuth
from flask_httpauth import HTTPTokenAuth
from flask_cors import CORS
from flask import Flask
from flask_restful_swagger_2 import Api

from actinia_core.core.common.config import global_config, DEFAULT_CONFIG_PATH

from actinia_api import API_VERSION, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Julia Haas, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2023, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


flask_app = Flask(__name__)
# allows endpoints with and without trailing slashes
flask_app.url_map.strict_slashes = False
CORS(flask_app)


flask_api = Api(
    flask_app,
    prefix=URL_PREFIX,
    api_version=API_VERSION,
    api_spec_url="/swagger",
    title=actinia_string,
    description=actinia_description,
    schemes=["https"],
    consumes=["application/gml+xml", "application/json"],
)

# authentication method
global_config.read(DEFAULT_CONFIG_PATH)
if global_config.KEYCLOAK_CONFIG_PATH:
    auth = HTTPTokenAuth(scheme="Bearer")
    flask_api._swagger_object["securityDefinitions"] = {
        "bearerAuth": {
            "type": "http",
            "scheme": "bearer",
            "bearerFormat": "JWT",
        }
    }
    # https://swagger.io/docs/specification/authentication/oauth2/
    flask_api._swagger_object["security"] = [
        {
            "OAuth2": {
                "type": "oauth2",
                # "authorizationUrl": "http://swagger.io/api/oauth/dialog",
                "tokenUrl": f"{global_config.KEYCLOAK_URL}/realms/"
                f"{global_config.KEYCLOAK_REALM}/protocol/openid-connect/"
                "token",
                "flow": "implicit",
                "scopes": {},
            }
        }
    ]
elif global_config.AUTHENTICATION:
    # Set the security definition in an unconventional way
    flask_api._swagger_object["securityDefinitions"] = {
        "basicAuth": {"type": "basic"}
    }
    flask_api._swagger_object["security"] = [{"basicAuth": []}]

    auth = HTTPBasicAuth()
else:
    # No authentication
    auth = HTTPBasicAuth()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Message logger test case

"""
import unittest
from actinia_core.core.common.config import Configuration
from actinia_core.core.messages_logger import MessageLogger

__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class MessageLoggerTestCase(unittest.TestCase):
    """
    This class tests the user interface, the creation and deletion
    of a user entry in a Redis database
    """

    def test_change_loglevel(self):
        c = Configuration()
        c.LOG_LEVEL = 4

        logger = MessageLogger(config=c, user_id="soeren")
        logger.debug("debug")
        logger.info("info")
        logger.warning("warning")
        logger.error("error")

        c = Configuration()
        c.LOG_LEVEL = 3

        logger = MessageLogger(config=c, user_id="soeren")
        logger.debug("debug")
        logger.info("info")
        logger.warning("warning")
        logger.error("error")

        c = Configuration()
        c.LOG_LEVEL = 2

        logger = MessageLogger(config=c, user_id="soeren")
        logger.debug("debug")
        logger.info("info")
        logger.warning("warning")
        logger.error("error")

        c = Configuration()
        c.LOG_LEVEL = 1

        logger = MessageLogger(config=c, user_id="soeren")
        logger.debug("debug")
        logger.info("info")
        logger.warning("warning")
        logger.error("error")

    def test_fluentd(self):
        c = Configuration()
        c.LOG_LEVEL = 4
        c.LOG_INTERFACE = "fluentd"

        logger = MessageLogger(config=c, user_id="soeren")
        logger.debug("debug")
        logger.info("info")
        logger.warning("warning")
        logger.error("error")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Log messages as debug, info, warning and error
"""

import time
import platform
from .redis_fluentd_logger_base import RedisFluentLoggerBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-present, Sören Gebbert and mundialis GmbH & Co. KG"
)


class MessageLogger(RedisFluentLoggerBase):
    """Message logger that puts are logs to stderr"""

    def __init__(self, config=None, user_id=None, fluent_sender=None):
        RedisFluentLoggerBase.__init__(
            self, config=config, user_id=user_id, fluent_sender=fluent_sender
        )

    def _log_message(self, log_level, message):
        node = platform.node()
        ctime = time.ctime()

        log_dict = {
            "node": node,
            "ctime": ctime,
            "user_id": self.user_id,
            "log_level": log_level,
            "status": "message",
            "message": message,
            "logger": "messages_logger",
        }

        self.send_to_logger(log_level, log_dict)

        # TODO: is this return value used anywhere?
        return message

    def debug(self, debug_message):
        if self.log_level >= 4:
            return self._log_message("DEBUG", debug_message)

    def info(self, info_message):
        if self.log_level >= 3:
            return self._log_message("INFO", info_message)

    def warning(self, warn_message):
        if self.log_level >= 2:
            return self._log_message("WARNING", warn_message)

    def error(self, error_message):
        if self.log_level >= 1:
            return self._log_message("ERROR", error_message)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2019 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Actinia resource test case base
"""
import atexit
import os
import signal
import time
from actinia_core.testsuite import ActiniaTestCaseBase, URL_PREFIX
from actinia_core.core.common.config import global_config
from actinia_core.endpoints import create_endpoints

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis GmbH & Co. KG"

# Create endpoints
create_endpoints()

redis_pid = None
server_test = False
custom_actinia_cfg = False

# If this environmental variable is set, then a real http request will be send
# instead of using the flask test_client.
if "ACTINIA_SERVER_TEST" in os.environ:
    server_test = bool(os.environ["ACTINIA_SERVER_TEST"])
# Set this variable to use a actinia config file in a docker container
if "ACTINIA_CUSTOM_TEST_CFG" in os.environ:
    custom_actinia_cfg = str(os.environ["ACTINIA_CUSTOM_TEST_CFG"])

base_url_data = "https://apps.mundialis.de/actinia_test_datasets"
additional_external_data = {
    "rio_json": f"{base_url_data}/rio.json",
    "brazil_json": f"{base_url_data}/brazil_polygon.json",
    "elev_ned_30m_tif": f"{base_url_data}/elev_ned_30m.tif",
    "elev_ned_30m_epsg4326_tif": f"{base_url_data}/elev_ned_30m_epsg4326.tif",
    "elev_ned_30m_nope_tif": f"{base_url_data}/elev_ned_30m_nope.tif",
    "polygon_gml": f"{base_url_data}/polygon.gml",
    "census_wake2000_gml": f"{base_url_data}/census_wake2000.gml",
    "census_wake2000_zip": f"{base_url_data}/census_wake2000.zip",
    "geology_30m_tif": f"{base_url_data}/geology_30m.tif",
    "geology_30m_zip": f"{base_url_data}/geology_30m.zip",
    "pointInBonn": f"{base_url_data}/pointInBonn.geojson",
}


def setup_environment():
    global redis_pid
    # Set the port to the test redis server
    global_config.REDIS_SERVER_SERVER = "localhost"
    global_config.REDIS_SERVER_PORT = 7000
    # Set the path to redis WORKER_LOGFILE
    # global_config.WORKER_LOGFILE = "/var/log/redis/redis"

    # home = os.getenv("HOME")

    # GRASS GIS
    # Setup the test environment
    global_config.GRASS_GIS_BASE = "/usr/local/grass/"
    global_config.GRASS_GIS_START_SCRIPT = "/usr/local/bin/grass"
    # global_config.GRASS_DATABASE= "/usr/local/grass_test_db"
    # global_config.GRASS_DATABASE = "%s/actinia/grass_test_db" % home
    global_config.GRASS_TMP_DATABASE = "/tmp"

    if server_test is False and custom_actinia_cfg is False:
        # Start the redis server for user and logging management
        redis_pid = os.spawnl(
            os.P_NOWAIT,
            "/usr/bin/redis-server",
            "common/redis.conf",
            "--port %i" % global_config.REDIS_SERVER_PORT,
        )
        time.sleep(1)

    if server_test is False and custom_actinia_cfg is not False:
        global_config.read(custom_actinia_cfg)


def stop_redis():
    if server_test is False:
        global redis_pid
        # Kill th redis server
        if redis_pid is not None:
            os.kill(redis_pid, signal.SIGTERM)


# Register the redis stop function
atexit.register(stop_redis)
# Setup the environment
setup_environment()


class ActiniaResourceTestCaseBase(ActiniaTestCaseBase):
    pass



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async Process test case
"""
import os
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

try:
    import actinia_stac_plugin

    no_stac_plugin = False
except Exception:
    no_stac_plugin = True

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Julia Haas"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"
__email__ = "info@mundialis.de"

process_chain_raster_import_export = {
    "list": [
        {
            "flags": "a",
            "id": "r_slope_aspect_1",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["elev_ned_30m_tif"],
                        "type": "raster",
                    },
                    "param": "elevation",
                    "value": "elev_ned_30m",
                },
                {"param": "format", "value": "degree"},
                {"param": "precision", "value": "DCELL"},
            ],
            "module": "r.slope.aspect",
            "outputs": [
                {
                    "export": {"format": "GTiff", "type": "raster"},
                    "param": "slope",
                    "value": "elev_ned_30m_slope",
                },
                {
                    "export": {"format": "GTiff", "type": "raster"},
                    "param": "aspect",
                    "value": "elev_ned_30m_aspect",
                },
            ],
            "overwrite": False,
            "verbose": False,
        },
        {
            "exe": "/bin/cat",
            "id": "cat_1",
            "params": [],
            "stdin": "r_slope_aspect_1::stderr",
        },
    ],
    "version": "1",
}

process_chain_raster_import_info = {
    "list": [
        {
            "id": "r_info",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["elev_ned_30m_tif"],
                        "type": "raster",
                    },
                    "param": "map",
                    "value": "elev_ned_30m",
                }
            ],
            "module": "r.info",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_raster_import_error_no_file = {
    "list": [
        {
            "id": "r_info",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_nope_tif"
                        ],
                        "type": "raster",
                    },
                    "param": "map",
                    "value": "elev_ned_30m",
                },
            ],
            "module": "r.info",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_vector_import_info = {
    "list": [
        {
            "id": "v_info",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["polygon_gml"],
                        "type": "vector",
                    },
                    "param": "map",
                    "value": "polygon",
                }
            ],
            "module": "v.info",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_sentinel_import_info = {
    "list": [
        {
            "id": "r_info",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B01",
                    },
                    "param": "map",
                    "value": "sentinel_map",
                }
            ],
            "module": "r.info",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_sentinel_import_univar = {
    "list": [
        {
            "id": "r_univar",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B01",
                    },
                    "param": "map",
                    "value": "sentinel_map",
                }
            ],
            "module": "r.univar",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_sentinel_import_stats = {
    "list": [
        {
            "id": "r_stats",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B01",
                    },
                    "param": "input",
                    "value": "sentinel_map",
                }
            ],
            "module": "r.stats",
            "flags": "a",
        }
    ],
    "version": "1",
}

process_chain_sentinel_import_error = {
    "list": [
        {
            "id": "r_stats",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138_NOPE",
                        "type": "sentinel2",
                        "sentinel_band": "B01",
                    },
                    "param": "input",
                    "value": "sentinel_map",
                }
            ],
            "module": "r.info",
            "flags": "g",
        }
    ],
    "version": "1",
}

process_chain_sentinel_import_export = {
    "list": [
        {
            "id": "importer_1",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B01",
                    },
                    "param": "map",
                    "value": "sentinel_map",
                },
                {
                    "import_descr": {
                        "source": additional_external_data["elev_ned_30m_tif"],
                        "type": "raster",
                    },
                    "param": "map",
                    "value": "elev_ned_30m",
                },
            ],
            "outputs": [
                {
                    "export": {"type": "raster", "format": "GTiff"},
                    "param": "map",
                    "value": "sentinel_map",
                },
                {
                    "export": {"type": "raster", "format": "GTiff"},
                    "param": "map",
                    "value": "elev_ned_30m",
                },
            ],
            "module": "importer",
        }
    ],
    "version": "1",
}


process_chain_sentinel_import_export_sentinel_ndvi = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B04",
                    },
                    "param": "map",
                    "value": "B04",
                },
                {
                    "import_descr": {
                        "source": "S2A_MSIL1C_20170212T104141_N0204_R008_"
                        "T31TGJ_20170212T104138",
                        "type": "sentinel2",
                        "sentinel_band": "B08",
                    },
                    "param": "map",
                    "value": "B08",
                },
            ],
        },
        {
            "id": "rmapcalc_1",
            "module": "r.mapcalc",
            "inputs": [
                {
                    "param": "expression",
                    "value": "NDVI = (B08 - B04)/(B08 + B04)",
                }
            ],
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {"type": "raster", "format": "GTiff"},
                    "param": "map",
                    "value": "NDVI",
                }
            ],
        },
    ],
    "version": "1",
}

process_chain_stac_import = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "stac.defaultStac.rastercube."
                        "landsat-8-l1-c1",
                        "type": "stac",
                        "semantic_label": "B1",
                        "extent": {
                            "spatial": {
                                "bbox": [[30.192, -16.369, 42.834, -0.264]]
                            },
                            "temporal": {
                                "interval": [["2021-09-09", "2021-09-12"]]
                            },
                        },
                        "filter": {},
                    },
                    "param": "map",
                    "value": "landsat-8-l1-c1",
                }
            ],
        }
    ],
    "version": 1,
}

process_chain_stac_filter_error_import = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "stac.STAC_Others.rastercube."
                        "sentinel-s2-l2a",
                        "type": "stac",
                        "semantic_label": "red",
                        "extent": {
                            "spatial": {"bbox": [[-180, -16.369, 90, -0.264]]},
                            "temporal": {
                                "interval": [["2023-09-09", "2022-09-12"]]
                            },
                        },
                        "filter": {},
                    },
                    "param": "map",
                    "value": "landsat-8-l1-c1",
                }
            ],
        }
    ],
    "version": 1,
}

process_chain_stac_source_error_import = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "sentinel-s2-l2a",
                        "type": "stac",
                        "semantic_label": "red",
                        "extent": {
                            "spatial": {
                                "bbox": [[30.192, -16.369, 42.834, -0.264]]
                            },
                            "temporal": {
                                "interval": [["2021-09-09", "2021-09-12"]]
                            },
                        },
                        "filter": {},
                    },
                    "param": "map",
                    "value": "landsat-8-l1-c1",
                }
            ],
        }
    ],
    "version": 1,
}

process_chain_raster_import_resample_resolution = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "value",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

process_chain_raster_import_resample_resolution_info = {
    "list": [
        {
            "id": "r_info",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "value",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
            "module": "r.info",
            "flags": "g",
            "stdout": {"id": "r_info", "format": "kv", "delimiter": "="},
        }
    ],
    "version": "1",
}

process_chain_raster_import_resample_resolution_error_resamp = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinea",
                        "resolution": "value",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

process_chain_raster_import_resample_resolution_error_resol = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "values",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

pc_raster_import_resample_resolution_error_val_missing = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "value",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

pc_raster_import_resample_resolution_error_val_not_float = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "value",
                        "resolution_value": "not float",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

pc_raster_import_resample_resolution_error_resol_not_set = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}

pc_raster_import_resample_resolution_error_resol_not_val = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data[
                            "elev_ned_30m_epsg4326_tif"
                        ],
                        "type": "raster",
                        "resample": "bilinear_f",
                        "resolution": "estimated",
                        "resolution_value": "100",
                    },
                    "param": "map",
                    "value": "elev_ned_100m",
                }
            ],
        }
    ],
    "version": 1,
}


class AsyncProcessTestCase(ActiniaResourceTestCaseBase):
    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_raster_import_export_sentinel_ndvi(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                process_chain_sentinel_import_export_sentinel_ndvi
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_raster_import_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sentinel_import_export),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_raster_import(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_raster_import_info),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_raster_import_nofile(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_raster_import_error_no_file),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_import_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_raster_import_export),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_vector_import(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_vector_import_info),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_sentinel_import_info(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sentinel_import_info),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_sentinel_import_univar(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sentinel_import_univar),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_sentinel_import_stats(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sentinel_import_stats),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_sentinel_import_error(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_sentinel_import_error),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    # Test for STAC
    @unittest.skipIf(no_stac_plugin, "STAC Plugin not installed")
    def test_stac_import(self):
        """
        Test of STAC collection import with http response 200
        """

        endpoint = URL_PREFIX + "/locations/nc_spm_08/processing_async_export"
        rv = self.server.post(
            endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_stac_import),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    @unittest.skipIf(no_stac_plugin, "STAC Plugin not installed")
    def test_stac_source_error_import(self):
        """
        Test of STAC collection import with http response 400,
        raising error on wrongly structured, undefined, or missing source ID.
        """
        endpoint = URL_PREFIX + "/locations/nc_spm_08/processing_async_export"
        rv = self.server.post(
            endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_stac_source_error_import),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    @unittest.skipIf(no_stac_plugin, "STAC Plugin not installed")
    def test_stac_source_filter_error_import(self):
        """
        Test of STAC collection import with http response 400,
        raising error on filtering parameter such as wrong temporal interval
        or wrong Spatial coordinates in bbox.

        """
        endpoint = URL_PREFIX + "/locations/nc_spm_08/processing_async_export"
        rv = self.server.post(
            endpoint,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_stac_filter_error_import),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 200
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_raster_import_resample_resolution),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_raster_import_resample_resolution_info(self):
        """
        Test resampling and resolution values for raster import by retrieving
        resampling and resolution info
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                process_chain_raster_import_resample_resolution_info
            ),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header
        )

        resample_val = "resample=bilinear_f"
        resolution_val = "100"

        r_info_results = resp["process_results"]["r_info"]
        r_import_process_log = resp["process_log"][1]

        self.assertEqual(
            r_info_results["ewres"],
            resolution_val,
            "'ewres' does not match set resolution",
        )
        self.assertEqual(
            r_info_results["nsres"],
            resolution_val,
            "'nsres' does not match set resolution %s" % resolution_val,
        )
        self.assertEqual(
            r_import_process_log["parameter"][3],
            resample_val,
            "'resampling method' does not match set resampling method %s"
            % resample_val,
        )

    def test_raster_import_resample_resolution_error_resamp(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, resample value not
        in options
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                process_chain_raster_import_resample_resolution_error_resamp
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution_error_resol(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, resolution value not
        in options
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                process_chain_raster_import_resample_resolution_error_resol
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution_error_val_missing(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, key "value" missing when
        resolution set to value
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                pc_raster_import_resample_resolution_error_val_missing
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution_error_val_not_float(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, value for "value" not
        convertible to float
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                pc_raster_import_resample_resolution_error_val_not_float
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution_error_resol_not_set(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, resolution not set when
        value set
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                pc_raster_import_resample_resolution_error_resol_not_set
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

    def test_raster_import_resample_resolution_error_resol_not_val(self):
        """
        Test raster import with defined resampling method and defined
        resolution method, with http response 400, resolution not set to
        "value" when value set
        """
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(
                pc_raster_import_resample_resolution_error_resol_not_val
            ),
            content_type="application/json",
        )

        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async Process Validation test case
"""
import os
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis GmbH & Co. KG"

# Module example for r.out.ascii with g.region adjustment and temporary file
# handling
process_chain_legacy = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT", "res": 10000},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "r.out.ascii",
        "inputs": {"input": "elevation@PERMANENT"},
        "outputs": {"output": {"name": "$file::out1"}},
    },
    3: {"executable": "/bin/cat", "parameters": ["$file::out1"]},
    4: {
        "module": "r.out.ascii",
        "inputs": {"input": "elevation@PERMANENT"},
        "outputs": {"output": {"name": "$file::out2"}},
    },
    5: {"executable": "/bin/cat", "parameters": ["$file::out2"]},
    6: {"executable": "/bin/cat", "stdin": "5::stdout"},
}

# Module  example for r.out.ascii with g.region adjustment and temporary file
# handling
process_chain_new = {
    "version": 1,
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished",
        "update": "http://0.0.0.0:5005/webhook/update",
    },
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [
                {"param": "raster", "value": "elevation@PERMANENT"},
                {"param": "res", "value": "10000"},
            ],
            "flags": "p",
            "verbose": True,
        },
        {
            "id": "2",
            "module": "r.out.ascii",
            "inputs": [{"param": "input", "value": "elevation@PERMANENT"}],
            "outputs": [
                {
                    "export": {"type": "file", "format": "TXT"},
                    "param": "output",
                    "value": "$file::out1",
                }
            ],
        },
        {"id": "3", "exe": "/bin/cat", "params": ["$file::out1"]},
        {
            "id": "4",
            "module": "r.out.ascii",
            "inputs": [{"param": "input", "value": "elevation@PERMANENT"}],
            "outputs": [
                {
                    "export": {"type": "file", "format": "TXT"},
                    "param": "output",
                    "value": "$file::out2",
                }
            ],
        },
        {"id": "5", "exe": "/bin/cat", "params": ["$file::out2"]},
        {"id": "6", "exe": "/bin/cat", "stdin": "5::stdout"},
    ],
}

# Import a Sentinel2A scene and compute the NDVI for a specific polygon
process_chain_ndvi = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "S2B_MSIL1C_20171010T131249_N0205_R081_"
                        "T26VPR_20171010T131243",
                        "type": "sentinel2",
                        "sentinel_band": "B04",
                    },
                    "param": "map",
                    "value": "B04",
                },
                {
                    "import_descr": {
                        "source": "S2B_MSIL1C_20171010T131249_N0205_R081_"
                        "T26VPR_20171010T131243",
                        "type": "sentinel2",
                        "sentinel_band": "B08",
                    },
                    "param": "map",
                    "value": "B08",
                },
            ],
        },
        {
            "id": "g_region_1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "B04"}],
            "flags": "g",
        },
        {
            "id": "rmapcalc_1",
            "module": "r.mapcalc",
            "inputs": [
                {
                    "param": "expression",
                    "value": "NDVI = float((B08 - B04)/(B08 + B04))",
                }
            ],
        },
        {
            "id": "r_univar_sentinel2",
            "module": "r.univar",
            "inputs": [{"param": "map", "value": "NDVI"}],
            "flags": "g",
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {"type": "raster", "format": "GTiff"},
                    "param": "map",
                    "value": "NDVI",
                }
            ],
        },
    ],
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished",
        "update": "http://0.0.0.0:5005/webhook/update",
    },
    "version": "1",
}

# Import a Sentinel2A scene and compute the NDVI for a specific polygon
# https://storage.googleapis.com/graas-geodata/rio.json
# https://gdh-data-sandbox.ams3.digitaloceanspaces.com/rio.json
# https://apps.mundialis.de/actinia_test_datasets/rio.json
process_chain_ndvi_landsat = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "LT52170762005240COA00",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
                {
                    "import_descr": {
                        "source": additional_external_data["rio_json"],
                        "type": "vector",
                    },
                    "param": "map",
                    "value": "polygon",
                },
            ],
        },
        {
            "id": "g_list",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "g_rename_1",
            "module": "g.rename",
            "inputs": [
                {
                    "param": "raster",
                    "value": "LT52170762005240COA00_dos1.4,RED",
                }
            ],
        },
        {
            "id": "g_rename_2",
            "module": "g.rename",
            "inputs": [
                {
                    "param": "raster",
                    "value": "LT52170762005240COA00_dos1.5,NIR",
                }
            ],
        },
        {
            "id": "g_region_1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "RED"}],
            "flags": "g",
        },
        {
            "id": "g_region_2",
            "module": "g.region",
            "inputs": [{"param": "vector", "value": "polygon"}],
            "flags": "g",
        },
        {
            "id": "r_mask",
            "module": "r.mask",
            "inputs": [{"param": "vector", "value": "polygon"}],
        },
        {
            "id": "rmapcalc_1",
            "module": "r.mapcalc",
            "inputs": [
                {
                    "param": "expression",
                    "value": "NDVI = float((NIR - RED)/(NIR + RED))",
                }
            ],
        },
        {
            "id": "r_univar_ndvi",
            "module": "r.univar",
            "inputs": [{"param": "map", "value": "NDVI"}],
            "flags": "g",
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {"type": "raster", "format": "GTiff"},
                    "param": "map",
                    "value": "NDVI",
                }
            ],
        },
    ],
    "webhooks": {
        "finished": "http://0.0.0.0:5005/webhook/finished",
        "update": "http://0.0.0.0:5005/webhook/update",
    },
    "version": "1",
}

process_chain_landsat = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": "LT52170762005240COA00",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
                {
                    "import_descr": {
                        "source": "LE70030041999205KIS00",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
                {
                    "import_descr": {
                        "source": "LC80050042013137LGN01",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
                {
                    "import_descr": {
                        "source": "LE72170762002288CUB00",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
                {
                    "import_descr": {
                        "source": "LT52170762007278COA00",
                        "type": "landsat",
                        "landsat_atcor": "dos1",
                    },
                    "param": "map",
                    "value": "ignored",
                },
            ],
        }
    ],
    "webhooks": {"finished": "http://0.0.0.0:5005/webhook/finished"},
    "version": "1",
}


class AsyncProcessValidationTestCase(ActiniaResourceTestCaseBase):
    def test_async_processing_legacy(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_legacy),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertEqual(len(resp["process_results"]), 6)

    def test_async_processing_new(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/process_chain_validation_sync",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_new),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertEqual(len(resp["process_results"]), 6)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_async_processing_new_ndvi(self):
        rv = self.server.post(
            URL_PREFIX
            + "/locations/latlong_wgs84/process_chain_validation_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_ndvi),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        print(resp)
        self.assertEqual(len(resp["process_results"]), 29)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_async_processing_new_ndvi_export_landsat(self):
        rv = self.server.post(
            URL_PREFIX
            + "/locations/latlong_wgs84/process_chain_validation_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_ndvi_landsat),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertEqual(len(resp["process_results"]), 26)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_async_processing_landsat(self):
        rv = self.server.post(
            URL_PREFIX
            + "/locations/latlong_wgs84/process_chain_validation_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_landsat),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.assertEqual(len(resp["process_results"]), 96)


if __name__ == "__main__":
    unittest.main()
